use strict;
use warnings;
package Git::Megapull::Source;
{
  $Git::Megapull::Source::VERSION = '0.101752';
}
# ABSTRACT: a source of cloneable git repository URIs


1;

__END__

=pod

=head1 NAME

Git::Megapull::Source - a source of cloneable git repository URIs

=head1 VERSION

version 0.101752

=head1 OVERVIEW

This will be a base class for sources for Git::Megapull.  Right now, it does
nothing.

See L<Git::Megapull>.

=head1 AUTHOR

Ricardo SIGNES <rjbs@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by Ricardo SIGNES.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
